/* Fianchetto 0.0 - players.c - 
   Copyright (c) 2003 Ignacio Garca Fernndez

   Fianchetto is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   Fianchetto is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Fianchetto; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.

   Contact Info: ignacio.garcia@uv.es
*/


#include"players.h"

#include<stdlib.h>


/******************************************************
 * These functions let us sort the set of players
 * according to Rule A.2 of FSR
 ******************************************************/

int compare_name(const t_player *p1,const t_player *p2)
{
  return 0;
}

int compare_title(const t_player *p1,const t_player *p2)
{
  int result;

  if(p1->title<p2->title) result = 1;
  else if(p1->title>p2->title) result = -1;
  else result = compare_name(p1,p2);

  return result;
}

int compare_rating(const t_player *p1,const t_player *p2)
{
  int result;

  if(p1->rating<p2->rating) result = 1;
  else if(p1->rating>p2->rating) result = -1;
  else result = compare_title(p1,p2);

  return result;
}

int compare_plyrs(const t_player *p1,const t_player *p2)
{
  int result;

  if(p1->score<p2->score) result = 1;
  else if(p1->score>p2->score) result = -1;
  else result = compare_rating(p1,p2);

  /* here more code for other parameters (eg. butchholz) can be added) */

  return result;
}

void sort_players(t_player *players,int numplayers)
{
  qsort(players,numplayers,sizeof(t_player),
	(int (*)(const void *,const void *)) compare_plyrs);
}


t_player *
read_player_file(FILE *f,int *n)
{

  char S[56],c[32];
  char ends[1];
  int i=0,j,no_end,field,beg_of_field;
  t_player *pl;

  ends[0] = '\0';

  /* first, we read the number of players */

  fscanf(f,"PLAYERS %d\n",n);

  pl = (t_player *) calloc(sizeof(t_player),*n);

  while(i<*n)
    {
      j=0;
      field=0;
      beg_of_field=0;
      fscanf(f,"%s\n",S);
      strcpy(c,S);
      no_end=1;
      clear_plyr(pl);

      while(field<NUM_FIELDS)
	{
	  while(no_end)
	    {
	      if(S[j]==*ends)
		{
		  no_end=0;
		  c[j] = *ends;
		  if(store_field(field,beg_of_field-j,&c[beg_of_field],&pl[i]))
		    {
		      fprintf(stderr,"Se produjo un error leyendo de fichero. El campo \"nombre\" de la lnea %d est vaco\n",i+2);
		      return NULL;
		    }
		  field++;
		}
	      else
		{
		  if(S[j]==':') /* Separator found */
		    {
		      c[j] = *ends;
		      if(store_field(field,beg_of_field-j,&c[beg_of_field],&pl[i]))
			{
			  fprintf(stderr,"Se produjo un error leyendo de fichero. El campo \"nombre\" de la lnea %d est vaco\n",i+2);
			  return NULL;
			}
		      field++;
		      beg_of_field=j+1;
		    }
		  j++;
		}
	    } /* The line ended */
	  if(field<NUM_FIELDS) printf("\tCampos insuficientes en la lnea %d",i+2);
	  field = NUM_FIELDS;
	}
      no_end = 1;
      i++;
    }
  fflush(stdout);
  return pl;
}



int
store_field(int f_num,int len,char *data,t_player *pl)
{
  switch(f_num)
    {
    case NAME:
      if(len)
	strcpy(pl->name,data);
      else
	{
	  fprintf(stderr,"Error. Campo %d vaco\n",f_num);
	  return 1;
	}
    case RATING:
      if(len)
	pl->rating = strtol(data,NULL, 10);
      else
	fprintf(stderr,"Error. Campo %d vaco\n",f_num);
      break;
    case TITLE:
      if(len)
	pl->title = strtol(data,NULL, 10);
      else
	fprintf(stderr,"Error. Campo %d vaco\n",f_num);
      break;
    }

  return 0;
}


void clear_plyr(t_player *pl)
{
  pl->rating     = 0;
  pl->Id         = 0;
  pl->score      = 0;
  pl->title      = 0;
  pl->games      = 0;
  pl->wins       = 0;
  pl->losses     = 0;
  pl->draws      = 0;
  pl->forfeits   = 0;
  pl->abuse      = 0;
  pl->updatetime = 0;
  pl->next       = NULL;
  pl->prev       = NULL;
}
